#pragma once
#ifndef LRTABLES_GENERATOR_H_YW3GIUNH
#define LRTABLES_GENERATOR_H_YW3GIUNH

#include "Parsodus/grammar.h"
#include "Parsodus/lrtables/table.h"

namespace pds {
namespace lr {

/**
 * Base class for LR (and derivative) table generators (such as SLR and LALR)
 * Parametrized on the type of item to be used in the configuration sets
 */
template <typename Item>
class Generator {
    public:
        /**
         * Constructor
         *
         * @param g The grammar to translate
         */
        Generator(const Grammar& g);

        /**
         * Generate an LRTable based on given grammar
         *
         * @returns An LR (or derivative) table for the grammar given at construction
         */
        LRTable generate();

    protected:
        /**
         * Indicate whether this specific algorithm needs to have the First and Follow sets generated
         */
        virtual bool needsFollowSet() = 0;

        std::set<std::string> first(std::string s);
        std::set<std::string> follow(std::string s);

    private:
        /**
         * Build the `First` set
         */
        void buildFirst();

        /**
         * Build the `Follow` set
         */
        void buildFollow();

        Grammar m_gram;
        std::map<std::string, std::set<std::string>> m_first;
        std::map<std::string, std::set<std::string>> m_follow;
};

} /* lr */
} /* pdf */

#endif /* LRTABLES_GENERATOR_H_YW3GIUNH */
